import SSGUtil
import markdown
from pygments.formatters import HtmlFormatter

html_skeleton = """<!doctype html>
<html lang="{lang}">
<head>
    <meta charset="utf-8">
    <title>{title}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="../assets/css/pygments.css">{stylesheet}
</head>
<body>
{content}
</body>
</html>
"""


# Returns None if no title is found
def get_txt_title(file_location):
    title = None
    first_3_lines = []
    i = 0

    # Read file line by line, up to line 3
    with open(file_location, "r", encoding="utf8") as input_file:
        while line := input_file.readline():

            if i == 3:
                break

            cleaned_line = line.rstrip()
            first_3_lines.append(cleaned_line)
            i += 1

    # Check if title criteria is met
    if len(first_3_lines) == 3:
        if first_3_lines[0] and not first_3_lines[1] and not first_3_lines[2]:
            title = first_3_lines[0]

    return title


# Returns html content, including title, if set
def generate_content(file_location, title):
    if not SSGUtil.is_file_accepted(file_location):
        return

    titled_format = "<h1>{}</h1>\n\n\n{}"
    content = ""

    with open(file_location, "r", encoding="utf8") as input_file:
        content = input_file.read()

    if file_location.endswith(".md"):
        # Formatter for code blocks
        formatter = HtmlFormatter(cssclass="codehilite")
        # Will not add if the asset already exists
        SSGUtil.add_asset("css", "pygments.css", formatter.get_style_defs())

        content = process_markdown(content)
    else:
        # Skip the first 3 lines if there's a title
        if title:
            content = content.split("\n", 3)[3]

        content = "<p>" + content
        content = content.replace("\n\n", "</p>\n\n<p>")
        content = content + "\n</p>"

        if title:
            content = titled_format.format(title, content)

    return content


def process_markdown(content):
    return markdown.markdown(
        content, extensions=["fenced_code", "codehilite", "tables"]
    )


# Inserts title, stylesheet, and content to html_skeleton, returns the result
def generate_html(lang, file_name, title, stylesheet, content):
    ss_tag = ""

    if stylesheet:
        ss_tag = '\n\t<link rel="stylesheet" href="{}">'.format(stylesheet)

    return html_skeleton.format(
        lang=lang,
        title=title if title else file_name,
        stylesheet=ss_tag,
        content=content if content else "",
    )



import argparse
import os
import shutil
import SSGParser
import SSGUtil


def main(args):
    parser = argparse.ArgumentParser(description="Static Site Generator")
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version="%(prog)s 0.1",
        help="Show program's version number and exit",
    )
    parser.add_argument("-i", "--input", help="Pass a file or folder of files")
    parser.add_argument("-s", "--stylesheet", help="URL to a stylesheet")
    parser.add_argument(
        "-l", "--lang", help="Language to be set in root html tag", default="en"
    )
    parser.add_argument("-c", "--config", help="Config file for arguments")
    args = parser.parse_args(args)

    if os.path.isdir(SSGUtil.OUTPUT_FOLDER):
        shutil.rmtree(SSGUtil.OUTPUT_FOLDER)

    lang = args.lang
    input = args.input
    stylesheet = args.stylesheet

    input, lang, stylesheet = SSGUtil.get_config(args.config, input, lang, stylesheet)

    files = []
    folder = ""
    is_folder = os.path.isdir(input)

    if is_folder:
        folder = input + "/"
        files = SSGUtil.get_accepted_files(folder)
    else:
        if SSGUtil.is_file_accepted(input):
            files.append(input)
        else:
            print("Invalid file type!")
            print(
                "Current accepted file types are: "
                + ", ".join(SSGUtil.ACCEPTED_FILE_TYPES)
            )

    SSGUtil.create_output_dir()

    for file in files:
        file_location = folder + file
        title = (
            SSGParser.get_txt_title(file_location)
            if file_location.endswith(".txt")
            else None
        )
        content = SSGParser.generate_content(file_location, title)
        # Make sure content was generated (file not skipped)
        if content:
            html = SSGParser.generate_html(lang, file, title, stylesheet, content)
            SSGUtil.output_to_file(file, html)


if __name__ == "__main__":
    import sys

    main(sys.argv[1:])



import json
import os

OUTPUT_FOLDER = "dist"
ACCEPTED_FILE_TYPES = [".txt", ".md"]
assets = []
asset_types = []


# Returns a list of files with acceptable file types by filtering
# all files in folder_name
def get_accepted_files(folder_name):
    all_files = os.listdir(folder_name)
    filtered_files = filter(lambda f: is_file_accepted(f), all_files)
    return list(filtered_files)


def is_file_accepted(filename):
    for type in ACCEPTED_FILE_TYPES:
        if filename.endswith(type):
            return True
    return False


def output_to_file(file, html):
    file_name = os.path.splitext(os.path.basename(file))[0]
    file_location = OUTPUT_FOLDER + "/" + file_name

    os.makedirs(file_location)

    with open(file_location + "/index.html", "w", encoding="utf8") as output_file:
        output_file.write(html)


def get_config(config, input, lang, stylesheet):
    # Check to see if config file exists.
    if config:
        with open(config) as f:
            try:
                data = json.load(f)
                if len(data) == 0:
                    print("\nConfig File is empty!\n")
                    exit(1)
            except (json.decoder.JSONDecodeError) as err:
                print("\nError parsing Config File: {0}\n".format(err))
                exit(1)
        # For each command from JSON config file, set local variables
        for i in data:
            if i == "input" or i == "i":
                input = data[i]
            elif i == "stylesheet" or i == "s":
                stylesheet = data[i]
            elif i == "lang" or i == "l":
                lang = data[i]
        if input is None:
            print("No input file specified")
            exit(1)
    return input, lang, stylesheet


def create_output_dir():
    # Create output folder if it doesn't exist
    if not os.path.isdir(OUTPUT_FOLDER):
        os.mkdir(OUTPUT_FOLDER)
        os.mkdir(OUTPUT_FOLDER + "/" + "assets")


def add_asset(asset_type, asset_name, asset_content):
    location = OUTPUT_FOLDER + "/assets/" + asset_type

    if asset_type not in asset_types:
        os.makedirs(location)
        asset_types.append(asset_type)

    if asset_name not in assets:
        with open(location + "/" + asset_name, "w") as asset_file:
            asset_file.write(asset_content)
        assets.append(asset_name)
